//*****************************************************************************
// File Name	: i2clcd.c
// 
// Title		: I2C -> LCD slave
// Revision		: 1.0
// Notes		: 
// Target MCU	: Atmel AVR series ATtiny2313
// Tool Chain	: WinAVR-20081205
// 
// Revision History:
// When			Who			Description of change
// -----------	-----------	-----------------------
// 07-Mar-2009	S.Takagi	Created the program
//*****************************************************************************

//----- Include Files ---------------------------------------------------------
#include <avr/io.h>		// include I/O definitions (port names, pin names, etc)
#include <avr/interrupt.h>	// include interrupt support
#include <avr/eeprom.h> 

#include "global.h"		// include our global settings
#include "lcd.h"
#include "usiTwiSlave.h"

#define ESC (0x1B)

static const uint8_t TWI_slaveAddress = 0x31;

EEMEM uint8_t LCD_CONTRAST = 0x80;
EEMEM uint8_t LCD_BACKLIT  = 0x80;



enum { S_NORMAL = 0, S_SEQUENCE };
enum { S_A0 = 10, S_A1 };
enum { S_B0 = 15, S_B1 };
enum { S_V0 = 20, S_V1 };
enum { S_S0 = 25, S_S1 };
enum { S_L0 = 30, S_L1, S_L2 };
enum { S_M0 = 40, S_M1, S_M2 };
enum { S_Q0 = 50, S_Q1, S_Q2, S_Q3 };


//----- Begin Code ------------------------------------------------------------
int main(void)
{
	static uint8_t rbuf[8], *p, temp;
	static uint8_t state = S_NORMAL;
	static uint8_t semaphore = 0xFF; // busy

	// I2C Slave initialization
	usiTwiSlaveInit( TWI_slaveAddress );

	// initialize LCD
	lcdInit();

	TCCR1A = (1<<COM1A1)|(0<<COM1A0)|(1<<COM1B1)|(0<<COM1B0)|(0<<WGM11)|(1<<WGM10);
	TCCR1B = (0<<WGM13)|(1<<WGM12)|(1<<CS10);
	OCR1A = eeprom_read_byte(&LCD_CONTRAST);
	OCR1B = eeprom_read_byte(&LCD_BACKLIT);
	DDRB |= (1<<PB3)|(1<<PB4);

	// any logos?
	// write something

	semaphore = 0x00; // available
	sei();
	while(1) {
	    if( usiTwiDataInReceiveBuffer() ) {
	        temp = usiTwiReceiveByte();

			switch (state) {
			case S_NORMAL:
				if(temp == ESC)
					state = S_SEQUENCE;
				else
					lcdDataWrite(temp);
				break;
			case S_SEQUENCE:
				p = rbuf;
				*p = temp;
				switch (temp) {
				case 'L': // L for Locate (X), (Y)
					state = S_L1;
					break;
				case 'a': // a for DDRAM address read
					usiTwiTransmitByte(0xFF); // currently not implemented
					state = S_NORMAL;
					break;
				case 'A': // A for DDRAM address set
					state = S_A1;
					break;
				case 'b': // a for Back Light parameter read
					usiTwiTransmitByte(OCR1BL);
					state = S_NORMAL;
					break;
				case 'v': // a for Vlc LCD Contrast parameter read
					usiTwiTransmitByte(OCR1AL);
					state = S_NORMAL;
					break;
				case 'B': // B for Back Light parameter set
					state = S_B1;
					break;
				case 'V': // V for Vlc LCD Contrast parameter set
					state = S_V1;
					break;
				case 'H': // H for cursor go HOME
					lcdHome();
					state = S_NORMAL;
					break;
				case 'C': // C for Clear LCD
					lcdClear();
					state = S_NORMAL;
					break;
				case 's': // s for read (semaphore)
					usiTwiTransmitByte(semaphore);
					state = S_NORMAL;
					break;
				case 'S': // S for write semaphore
					state = S_S1;
					break;
				case 'M': // M for Custom Character Mapping
					state = S_M1;
					break;
				case 'W': // W for write OCR to EEPROM
					eeprom_write_byte (&LCD_CONTRAST, OCR1AL);
					eeprom_write_byte (&LCD_BACKLIT, OCR1BL);
					state = S_NORMAL;
					break;
				case 'Z': // Z for Custom Char Mapping for Progress Bar initialization
					lcdMapCustomChar(0,0);
					lcdMapCustomChar(1,1);
					lcdMapCustomChar(2,2);
					lcdMapCustomChar(3,3);
					lcdMapCustomChar(4,4);
					lcdMapCustomChar(5,5);
					state = S_NORMAL;
					break;
				}
				break;
			case S_A1:
				lcdDdramAddr(temp);
				state = S_NORMAL;
				break;
			case S_L1:
				*++p = temp;
				state = S_L2;
				break;
			case S_L2:
				*++p = temp;
				lcdGotoXY(rbuf[1], rbuf[2]);
				state = S_NORMAL;
				break;
			case S_S1:
				if (semaphore == temp)
					semaphore = 0x00;
				else if (semaphore == 0x00)
					semaphore = temp;
				state = S_NORMAL;
				break;
			case S_B1:
				//*++p = temp;
				OCR1B = temp;
				state = S_NORMAL;
				break;
			case S_V1:
				//*++p = temp;
				OCR1A = temp;
				state = S_NORMAL;
				break;
			case S_M1:
				*++p = temp;
				state = S_M2;
				break;
			case S_M2:
				*++p = temp;
				lcdMapCustomChar(rbuf[1], rbuf[2]);
				state = S_NORMAL;
				break;
			}
	    }
	}

	return 0;
}
